# THIS FILE IS AUTOMATICALLY GENERATED BY   script/development/namespaces_regenerate.pl
# DO NOT EDIT THIS FILE DIRECTLY!!!  please put all changes in namespaces_regenerate.pl
## no critic qw(Capitalization ProhibitMultiplePackages ProhibitReusedNames)  # SYSTEM DEFAULT 3: allow multiple & lower case package names
package    # hide from PAUSE indexing
    rperlnamespaces_generated;
use strict;
use warnings;
our $VERSION = 0.001_000;

## no critic qw(ProhibitParensWithBuiltins ProhibitNoisyQuotes)  # SYSTEM SPECIAL 3: allow auto-generated code

$rperlnamespaces_generated::NONCOMPILED = undef;
$rperlnamespaces_generated::NONCOMPILED = {
    'Acme::'       => 1,
    'Algorithm::'  => 1,
    'Archive::'    => 1,
    'Array::'      => 1,
    'Attribute::'  => 1,
    'Authen::'     => 1,
    'Benchmark::'  => 1,
    'CGI::'        => 1,
    'Compress::'   => 1,
    'Cookie::'     => 1,
    'Cpanel::'     => 1,
    'Crypt::'      => 1,
    'DBD::'        => 1,
    'DBI::'        => 1,
    'Date::'       => 1,
    'Excel::'      => 1,
    'FFI::'        => 1,
    'Geo::'        => 1,
    'HTTP::'       => 1,
    'Hash::'       => 1,
    'Hook::'       => 1,
    'IS::'         => 1,
    'Import::'     => 1,
    'LWP::'        => 1,
    'Mail::'       => 1,
    'MooX::'       => 1,
    'MooseX::'     => 1,
    'Net::'        => 1,
    'Number::'     => 1,
    'Opcode::'     => 1,
    'Plack::'      => 1,
    'RPC::'        => 1,
    'Sereal::'     => 1,
    'StackTrace::' => 1,
    'Statistics::' => 1,
    'Stream::'     => 1,
    'TAP::'        => 1,
    'Throwable::'  => 1,
    'Tree::'       => 1,
    'URI::'        => 1,
    'Unicode::'    => 1,
    'WWW::'        => 1,
    'XML::'        => 1,
    'ZMQ::'        => 1,
    'autouse::'    => 1,
    'metaclass::'  => 1,
    'subs::'       => 1
};

$rperlnamespaces_generated::CORE = undef;
$rperlnamespaces_generated::CORE = {
    '!'             => 1,
    'B::'           => 1,
    'CORE::'        => 1,
    'Carp::'        => 1,
    'DB::'          => 1,
    'Data::'        => 1,
    'DynaLoader::'  => 1,
    'ENV'           => 1,
    'ERRNO'         => 1,
    'English::'     => 1,
    'Errno::'       => 1,
    'Exporter::'    => 1,
    'INC'           => 1,
    'IO::'          => 1,
    'Internals::'   => 1,
    'OS_ERROR'      => 1,
    'PerlIO::'      => 1,
    'Regexp::'      => 1,
    'SIG'           => 1,
    'Scalar::'      => 1,
    'Tie::'         => 1,
    'UNIVERSAL::'   => 1,
    'XSLoader::'    => 1,
    'bytes::'       => 1,
    'constant::'    => 1,
    'main::'        => 1,
    'mro::'         => 1,
    'overload::'    => 1,
    'overloading::' => 1,
    're::'          => 1,
    'strict::'      => 1,
    'threads::'     => 1,
    'utf8::'        => 1,
    'version::'     => 1,
    'warnings::'    => 1
};

$rperlnamespaces_generated::RPERL_DEPS = undef;
$rperlnamespaces_generated::RPERL_DEPS = {
    'Alien::'             => 1,
    'Authen::'            => 1,
    'AutoLoader::'        => 1,
    'AutoSplit::'         => 1,
    'BSON::'              => 1,
    'CPAN::'              => 1,
    'Capture::'           => 1,
    'Class::'             => 1,
    'Clone::'             => 1,
    'Config::'            => 1,
    'Config_git::'        => 1,
    'Config_heavy::'      => 1,
    'Cross::'             => 1,
    'Cwd::'               => 1,
    'DateTime::'          => 1,
    'Devel::'             => 1,
    'Digest::'            => 1,
    'DirHandle::'         => 1,
    'Does::'              => 1,
    'Dos::'               => 1,
    'EPOC::'              => 1,
    'Email::'             => 1,
    'Encode::'            => 1,
    'Env::'               => 1,
    'Error::'             => 1,
    'Eval::'              => 1,
    'Exception::'         => 1,
    'ExtUtils::'          => 1,
    'FFI::'               => 1,
    'FakeLocale::'        => 1,
    'Fcntl::'             => 1,
    'File::'              => 1,
    'FileHandle::'        => 1,
    'Filter::'            => 1,
    'FindBin::'           => 1,
    'Getopt::'            => 1,
    'HTML::'              => 1,
    'I18N::'              => 1,
    'IPC::'               => 1,
    'Inline::'            => 1,
    'JSON::'              => 1,
    'Lingua::'            => 1,
    'List::'              => 1,
    'Locale::'            => 1,
    'Log::'               => 1,
    'MIME::'              => 1,
    'MM::'                => 1,
    'MRO::'               => 1,
    'MY::'                => 1,
    'Mango::'             => 1,
    'Math::'              => 1,
    'Method::'            => 1,
    'Module::'            => 1,
    'MongoDB::'           => 1,
    'Moo::'               => 1,
    'Moose::'             => 1,
    'Mouse::'             => 1,
    'Mozilla::'           => 1,
    'Net::'               => 1,
    'Object::'            => 1,
    'POSIX::'             => 1,
    'PPI::'               => 1,
    'PPIx::'              => 1,
    'Package::'           => 1,
    'PadWalker::'         => 1,
    'Params::'            => 1,
    'Parse::'             => 1,
    'Path::'              => 1,
    'Perl::'              => 1,
    'Pod::'               => 1,
    'Readonly::'          => 1,
    'Ref::'               => 1,
    'Role::'              => 1,
    'SDL::'               => 1,
    'SDL_perl::'          => 1,
    'SDLx::'              => 1,
    'Safe::'              => 1,
    'SelectSaver::'       => 1,
    'SelfLoader::'        => 1,
    'Socket::'            => 1,
    'Specio::'            => 1,
    'Storable::'          => 1,
    'String::'            => 1,
    'Sub::'               => 1,
    'Symbol::'            => 1,
    'Syntax::'            => 1,
    'Sys::'               => 1,
    'Term::'              => 1,
    'Test2::'             => 1,
    'Test::'              => 1,
    'Text::'              => 1,
    'Tie::'               => 1,
    'Time::'              => 1,
    'Try::'               => 1,
    'Type::'              => 1,
    'Types::'             => 1,
    'Unicode::'           => 1,
    'VMS::'               => 1,
    'Validation::'        => 1,
    'Variable::'          => 1,
    'Win32::'             => 1,
    '_charnames::'        => 1,
    'attributes::'        => 1,
    'auto::'              => 1,
    'base::'              => 1,
    'boolean::'           => 1,
    'bytes_heavy::'       => 1,
    'charnames::'         => 1,
    'deprecate::'         => 1,
    'f845a9c1ac41be33::'  => 1,
    'feature::'           => 1,
    'fields::'            => 1,
    'flock::'             => 1,
    'if::'                => 1,
    'integer::'           => 1,
    'isn::'               => 1,
    'locale::'            => 1,
    'maybe::'             => 1,
    'namespace::'         => 1,
    'next::'              => 1,
    'parent::'            => 1,
    'psSnake::'           => 1,
    're::'                => 1,
    'ref::'               => 1,
    'strict::'            => 1,
    'strictures::'        => 1,
    'swig_runtime_data::' => 1,
    'unicore::'           => 1,
    'utf8_heavy::'        => 1,
    'vars::'              => 1,
    'warnings::'          => 1
};

# DEV NOTE, CORRELATION #rp051: hard-coded list of RPerl data types and data structures [AUTOMATICALLY GENERATED BY script/development/namespaces_regenerate.pl]
# DEV NOTE, CORRELATION #rp054: auto-generation of OO property accessors/mutators checks the auto-generated RPerl type list for base data types to determine if the entire data structure can be returned by setting ($return_whole = 1)
$rperlnamespaces_generated::RPERL = undef;
$rperlnamespaces_generated::RPERL = {
    'RPerl::'                                 => 1,
    'RPerl::Support::'                        => 1,
    'RPerl::Support::MongoDB::'               => 1,
    'array::'                                 => 1,
    'arrayref::'                              => 1,
    'arrayref_arrayref::'                     => 1,
    'arrayref_arrayref_arrayref::'            => 1,
    'arrayref_arrayref_hashref::'             => 1,
    'arrayref_hashref::'                      => 1,
    'arrayref_hashref_hashref::'              => 1,
    'boolean::'                               => 1,
    'character::'                             => 1,
    'character_arrayref::'                    => 1,
    'character_hashref::'                     => 1,
    'coderef::'                               => 1,
    'constant_integer::'                      => 1,
    'constant_number::'                       => 1,
    'constant_sse_number_pair::'              => 1,
    'constant_unsigned_integer::'             => 1,
    'filehandleref::'                         => 1,
    'gmp_integer::'                           => 1,
    'gsl_matrix::'                            => 1,
    'hash::'                                  => 1,
    'hashref::'                               => 1,
    'hashref_arrayref::'                      => 1,
    'hashref_arrayref_hashref::'              => 1,
    'hashref_hashref::'                       => 1,
    'hashref_hashref_hashref::'               => 1,
    'integer::'                               => 1,
    'integer_arrayref::'                      => 1,
    'integer_arrayref_arrayref::'             => 1,
    'integer_arrayref_arrayref_arrayref::'    => 1,
    'integer_arrayref_arrayref_hashref::'     => 1,
    'integer_arrayref_hashref::'              => 1,
    'integer_arrayref_hashref_hashref::'      => 1,
    'integer_hashref::'                       => 1,
    'integer_hashref_arrayref_hashref::'      => 1,
    'integer_hashref_hashref::'               => 1,
    'integer_hashref_hashref_hashref::'       => 1,
    'method::'                                => 1,
    'number::'                                => 1,
    'number_arrayref::'                       => 1,
    'number_arrayref_arrayref::'              => 1,
    'number_arrayref_arrayref_arrayref::'     => 1,
    'number_arrayref_arrayref_hashref::'      => 1,
    'number_arrayref_hashref::'               => 1,
    'number_arrayref_hashref_hashref::'       => 1,
    'number_hashref::'                        => 1,
    'number_hashref_arrayref_hashref::'       => 1,
    'number_hashref_hashref::'                => 1,
    'number_hashref_hashref_hashref::'        => 1,
    'object::'                                => 1,
    'object_arrayref::'                       => 1,
    'object_hashref::'                        => 1,
    'perlapinames_generated::'                => 1,
    'rperl::'                                 => 1,
    'rperlgmp::'                              => 1,
    'rperlgsl::'                              => 1,
    'rperlnames::'                            => 1,
    'rperlnamespaces::'                       => 1,
    'rperlnamespaces_generated::'             => 1,
    'rperloperations::'                       => 1,
    'rperloptions::'                          => 1,
    'rperlrules::'                            => 1,
    'rperlsse::'                              => 1,
    'rperltypes::'                            => 1,
    'rperltypesconv::'                        => 1,
    'rperltypessizes::'                       => 1,
    'scalartype::'                            => 1,
    'scalartype_arrayref::'                   => 1,
    'scalartype_arrayref_arrayref::'          => 1,
    'scalartype_arrayref_arrayref_arrayref::' => 1,
    'scalartype_arrayref_arrayref_hashref::'  => 1,
    'scalartype_arrayref_hashref_hashref::'   => 1,
    'scalartype_hashref::'                    => 1,
    'scalartype_hashref_arrayref_hashref::'   => 1,
    'scalartype_hashref_hashref::'            => 1,
    'scalartype_hashref_hashref_hashref::'    => 1,
    'sse_number_pair::'                       => 1,
    'string::'                                => 1,
    'string_arrayref::'                       => 1,
    'string_arrayref_arrayref::'              => 1,
    'string_arrayref_arrayref_arrayref::'     => 1,
    'string_arrayref_arrayref_hashref::'      => 1,
    'string_arrayref_hashref::'               => 1,
    'string_arrayref_hashref_hashref::'       => 1,
    'string_hashref::'                        => 1,
    'string_hashref_arrayref_hashref::'       => 1,
    'string_hashref_hashref::'                => 1,
    'string_hashref_hashref_hashref::'        => 1,
    'unknown::'                               => 1,
    'unsigned_integer::'                      => 1,
    'void::'                                  => 1
};

$rperlnamespaces_generated::RPERL_FILES = undef;
$rperlnamespaces_generated::RPERL_FILES = {
    'RPerl.pm'                             => 1,
    'RPerl/AfterSubclass.pm'               => 1,
    'RPerl/CompileUnit/Module/Class.pm'    => 1,
    'RPerl/Config.pm'                      => 1,
    'RPerl/Exporter.pm'                    => 1,
    'RPerl/HelperFunctions_cpp.pm'         => 1,
    'RPerl/Inline.pm'                      => 1,
    'RPerl/Support.pm'                     => 1,
    'RPerl/Support/MongoDB.pm'             => 1,
    'RPerl\\AfterSubclass.pm'              => 1,
    'RPerl\\CompileUnit\\Module\\Class.pm' => 1,
    'RPerl\\Config.pm'                     => 1,
    'RPerl\\Exporter.pm'                   => 1,
    'RPerl\\HelperFunctions_cpp.pm'        => 1,
    'RPerl\\Inline.pm'                     => 1,
    'RPerl\\Support.pm'                    => 1,
    'RPerl\\Support\\MongoDB.pm'           => 1,
    'perlapinames_generated.pm'            => 1,
    'rperlgmp.pm'                          => 1,
    'rperlgsl.pm'                          => 1,
    'rperlnames.pm'                        => 1,
    'rperlnamespaces.pm'                   => 1,
    'rperlnamespaces_generated.pm'         => 1,
    'rperloperations.pm'                   => 1,
    'rperloptions.pm'                      => 1,
    'rperlrules.pm'                        => 1,
    'rperlsse.pm'                          => 1,
    'rperltypes.pm'                        => 1,
    'rperltypesconv.pm'                    => 1,
    'rperltypessizes.pm'                   => 1
};

1;
