#ifndef __CPP__INCLUDED__rperlgsl_h
#define __CPP__INCLUDED__rperlgsl_h 0.001_000

#include <RPerl/DataStructure/GSLMatrix.cpp>  // -> GSLMatrix.h
#include <RPerl/Operation/Expression/Operator/GSLFunctions.cpp>  // -> GSLFunctions.h

#endif
