# [[[ PREPROCESSOR ]]]
# ...

# [[[ HEADER ]]]
use RPerl;
package RPerl::CompileUnit::Module::Package::Template;
use strict;
use warnings;
our $VERSION = 0.001_000;

# [[[ EXPORTS ]]]
# ...

# [[[ CRITICS ]]]
# ...

# [[[ INCLUDES ]]]
# ...

# [[[ CONSTANTS ]]]
# ...

# [[[ SUBROUTINES ]]]
# ...

1;  # end of package


# [[[ ADDITIONAL PACKAGES ]]]

# [[[ HEADER ]]]
# <<< CHANGE_ME: replace with real class name >>>
use RPerl;
package RPerl::CompileUnit::Module::Class::TemplateAdditional;
use strict;
use warnings;
our $VERSION = 0.001_000;

# [[[ EXPORTS ]]]
# ...

# [[[ CRITICS ]]]
# ...

# [[[ INCLUDES ]]]
# ...

# [[[ CONSTANTS ]]]
# ...

# [[[ SUBROUTINES ]]]
# ...

1;  # end of package 